///////////////////////////////////////////////////////////////////////////////
// Created on: April 2000
// Created by: Ronald Hordijk (hk@dgmr.nl)
//
// This is the class where the fractal is calculated.
// Space is divided into rectangular cells. On the points of the cell the julia value
// is calulated. And the surface in the cell where the value equals the border value
// is generated and send to facelist.
//
// Because the calculation of the fractal can take a very long time this code was
// moved from the main application to a seperate thread so that the use does not have
// to wait for calculation to finish.
/////////////////////////////////////////////////////////////////////////////////
unit CalcThread;

interface

uses
  Classes, SysUtils, Forms, FaceList, JuliaCalc, Coordinates, Vector;

type
  TCalcThread = class(TThread)
  private
    FaceList:        TFaceList;
    Coordinates:     TCoordinates;
    CalcParam:       TCalcParam;
    JuliaCalculator: TJuliaCalculator;
    JuliaCalcFunc:   TJuliaCalcFunc;
    Form:            TForm;
    ProgressText:    String;

    procedure CreateIsoSurfFromTetra(p1, p2, p3, p4: TVector3V;
      IsoValue: Double);
    procedure ShowProgress;
  protected
    procedure Execute; override;
  public
    constructor Create(AFaceList: TFaceList; ACoordinates: TCoordinates;
                        ACalcParam: TCalcParam; AForm: TForm);
  end;

implementation

uses
  Mainform;

/////////////////////////////////////////////////////////////////////////////////
constructor TCalcThread.Create(AFaceList: TFaceList;
            ACoordinates: TCoordinates; ACalcParam: TCalcParam; AForm: TForm);
begin          
  FaceList        := AFaceList;
  Coordinates     := ACoordinates;
  CalcParam       := ACalcParam;
  JuliaCalculator := TJuliaCalculator.Create;
  JuliaCalcFunc   := JuliaCalculator.SetParameters(CalcParam);
  Form            := AForm;

  priority        := tpHigher;

  inherited Create(False);
end;

/////////////////////////////////////////////////////////////////////////////////
procedure TCalcThread.CreateIsoSurfFromTetra(p1, p2, p3, p4 :TVector3V; IsoValue: Double);
var
  t: TVector3V;
begin
  // sort the values from Low to High
  if (p4[3] <= p3[3]) and (p4[3] <= p2[3]) and (p4[3] < p1[3]) then begin
    t := p1; p1 := p4; p4 := t;
  end else begin
    if (p3[3] <= p2[3]) and (p3[3] < p1[3]) then begin
      t := p1; p1 := p3; p3 := t;
    end else begin
      if p2[3] < p1[3] then begin
        t := p1; p1 := p2; p2 := t;
      end;
    end;
  end;

  if (p4[3] <= p3[3]) and (p4[3] < p2[3]) then begin
    t := p2; p2 := p4; p4 := t;
  end else begin
    if p3[3] < p2[3] then begin
      t := p2; p2 := p3; p3 := t;
    end;
  end;

  if p4[3] < p3[3] then begin
    t := p3; p3 := p4; p4 := t;
  end;

  // all values higher or all lower than the iso value so no iso-faces
  if p4[3] <= IsoValue then
    Exit;

  if p1[3] >= IsoValue then
    Exit;

  if (p2[3] > IsoValue) then begin  // Border between P1 and P2
    FaceList.AddPolygon(
      GetValuePoint(p1, p2, IsoValue),
      GetValuePoint(p1, p3, IsoValue),
      GetValuePoint(p1, p4, IsoValue),
      Vector3Sub(Vector3(p4),Vector3(p1)));
  end else if (p3[3] > IsoValue) then begin // Border between P2 and P3
    FaceList.AddPolygon(
      GetValuePoint(p1, p3, IsoValue),
      GetValuePoint(p2, p3, IsoValue),
      GetValuePoint(p2, p4, IsoValue),
      Vector3Sub(Vector3(p3),Vector3(p1)));
    FaceList.AddPolygon(
      GetValuePoint(p2, p4, IsoValue),
      GetValuePoint(p1, p3, IsoValue),
      GetValuePoint(p1, p4, IsoValue),
      Vector3Sub(Vector3(p4), Vector3(p1)));
  end else begin // Border between P3 and P4
    FaceList.AddPolygon(
      GetValuePoint(p2, p4, IsoValue),
      GetValuePoint(p3, p4, IsoValue),
      GetValuePoint(p1, p4, IsoValue),
      Vector3Sub(Vector3(p4),Vector3(p1)));
  end;
end;

/////////////////////////////////////////////////////////////////////////////////
procedure TCalcThread.ShowProgress;
begin
  TfrmMain(Form).StatusBar.Panels[0].Text := ProgressText;
  TfrmMain(Form).StatusBar.Repaint;
end;

/////////////////////////////////////////////////////////////////////////////////
procedure TCalcThread.Execute;
var
  i,j,k: integer;
  reslist: Array of array of array of TVector3V;
begin
  FaceList.StartFill(Coordinates.GridSize - 1,
                     Coordinates.GridSize - 1,
                     Coordinates.GridSize - 1);

  SetLength(reslist, 2, Coordinates.GridSize, Coordinates.GridSize);

  for i:= 0 to Coordinates.GridSize - 1 do begin
    ProgressText := Format( '%d%% - %d Polygons - %d Points',
                                 [(100 *i) div (Coordinates.GridSize - 1),
                                      FaceList.FaceCount, FaceList.PointCount]);
    Synchronize(ShowProgress);

    // first Calculate the values
    for j:= 0 to Coordinates.GridSize - 1 do begin
      for k:= 0 to Coordinates.GridSize - 1 do begin
        if CalcParam.Border then begin
          if JuliaCalcFunc(Coordinates.IndexToJuliaCoord(i, j, k)) > CalcParam.NrInter  then
            reslist[i mod 2][j][k] := Vector3V(Coordinates.IndexToViewCoord(i, j, k), CalcParam.NrInter + 1)
          else
            reslist[i mod 2][j][k] := Vector3V(Coordinates.IndexToViewCoord(i, j, k), CalcParam.NrInter - 1);
        end else begin
          reslist[i mod 2][j][k] := Vector3V(Coordinates.IndexToViewCoord(i, j, k),
                                             JuliaCalcFunc(Coordinates.IndexToJuliaCoord(i, j, k)) - 0.1);
        end;
        if Terminated then
          break;
      end;
      if Terminated then
        break;
    end;

    if Terminated then
      break;

    if i = 0 then
      Continue;

    for j:= 0 to Coordinates.GridSize - 2 do begin
      FaceList.CellChange(i , j, 0);
      for k:= 0 to Coordinates.GridSize - 2 do begin
       if (i+j+k) mod 2 = 0 then begin
         CreateIsoSurfFromTetra(reslist[i     mod 2, j  , k  ],
                                reslist[i     mod 2, j+1, k  ],
                                reslist[i     mod 2, j  , k+1],
                                reslist[(i+1) mod 2, j  , k  ], CalcParam.NrInter);
         CreateIsoSurfFromTetra(reslist[(i+1) mod 2, j+1, k  ],
                                reslist[i     mod 2, j+1, k  ],
                                reslist[(i+1) mod 2, j  , k  ],
                                reslist[(i+1) mod 2, j+1, k+1], CalcParam.NrInter);
         CreateIsoSurfFromTetra(reslist[i     mod 2, j+1, k+1],
                                reslist[(i+1) mod 2, j+1, k+1],
                                reslist[i     mod 2, j  , k+1],
                                reslist[i     mod 2, j+1, k  ], CalcParam.NrInter);
         CreateIsoSurfFromTetra(reslist[(i+1) mod 2, j  , k+1],
                                reslist[(i+1) mod 2, j+1, k+1],
                                reslist[i     mod 2, j  , k+1],
                                reslist[(i+1) mod 2, j  , k  ], CalcParam.NrInter);
         CreateIsoSurfFromTetra(reslist[i     mod 2, j+1, k  ],
                                reslist[(i+1) mod 2, j+1, k+1],
                                reslist[i     mod 2, j  , k+1],
                                reslist[(i+1) mod 2, j  , k  ], CalcParam.NrInter);
       end else begin
         CreateIsoSurfFromTetra(reslist[i     mod 2, j+1, k  ],
                                reslist[i     mod 2, j  , k  ],
                                reslist[(i+1) mod 2, j+1, k  ],
                                reslist[i     mod 2, j+1, k+1], CalcParam.NrInter);
         CreateIsoSurfFromTetra(reslist[(i+1) mod 2, j  , k  ],
                                reslist[i     mod 2, j  , k  ],
                                reslist[(i+1) mod 2, j+1, k  ],
                                reslist[(i+1) mod 2, j  , k+1], CalcParam.NrInter);
         CreateIsoSurfFromTetra(reslist[i     mod 2, j  , k+1],
                                reslist[i     mod 2, j  , k  ],
                                reslist[i     mod 2, j+1, k+1],
                                reslist[(i+1) mod 2, j  , k+1], CalcParam.NrInter);
         CreateIsoSurfFromTetra(reslist[(i+1) mod 2, j+1, k+1],
                                reslist[(i+1) mod 2, j+1, k  ],
                                reslist[i     mod 2, j+1, k+1],
                                reslist[(i+1) mod 2, j  , k+1], CalcParam.NrInter);
         CreateIsoSurfFromTetra(reslist[i     mod 2, j  , k  ],
                                reslist[(i+1) mod 2, j+1, k  ],
                                reslist[i     mod 2, j+1, k+1],
                                reslist[(i+1) mod 2, j  , k+1], CalcParam.NrInter);
        end;
        if Terminated then
          break;
      end;
      if Terminated then
        break;
    end;

  end;
  FaceList.EndFill;

  ProgressText := Format( '%d Polygons - %d Points',
                               [FaceList.FaceCount, FaceList.PointCount]);
  Synchronize(ShowProgress);

  Form.Invalidate;

  JuliaCalculator.Free;
end;

end.
