///////////////////////////////////////////////////////////////////////////////
// Created on: April 2000
// Created by: Ronald Hordijk (hk@dgmr.nl)
//
// This unit contains a structure (class) containing coordinate information.
// There are 3 coordinate systems Julia, viewing and grid, with some conversion functions.
//
// Julia coordinates are the coordinates on which the Julia values are calculated
// View coordinates are coordinates use for viewing. To make the presentation of the data
//   easier the julia coordinate are translates to a box for -1 to 1 on all axes.
// Grid coordinates are the indeze of the threedimenional grid.
///////////////////////////////////////////////////////////////////////////////

unit Coordinates;

interface

uses
  Vector;

type
  TCoordinates = class
    Start:     TVector3;   // Start Point in Julia coordinates
    Stop:      TVector3;   // Stop Point in Julia coordinates
    Step:      TVector3;   // Width of a cell in julia Coordinates
    NewStart:  TVector3;   // New Start Point in Julia coordinates
    NewStop:   TVector3;   // Stop Point in Julia coordinates
    StartView: TVector3;   // Start point in viewing Coordinates
    StepView:  TVector3;   // Width of a cell in viewing Coordinates
    GridSize:  Integer;    // Number of cells in X,Y en Z direction

    procedure RecalcCoordTrans;
    function  IndexToViewCoord(IndexX, IndexY, indexZ: Integer): TVector3;
    function  IndexToJuliaCoord(IndexX, IndexY, indexZ: Integer): TVector3;
    function  JuliaToViewCoord(V: TVector3): TVector3;
  end;

implementation

///////////////////////////////////////////////////////////////////////////////
procedure TCoordinates.RecalcCoordTrans;
var
  maxdist: double;
begin
  Step[0] := (Stop[0] - Start[0])/GridSize;
  Step[1] := (Stop[1] - Start[1])/GridSize;
  Step[2] := (Stop[2] - Start[2])/GridSize;

  Maxdist := (Stop[0] - Start[0]);
  if (Stop[1] - Start[1]) > maxdist then
    maxDist := (Stop[1] - Start[1]);
  if (Stop[2] - Start[2]) > maxdist then
    maxDist := (Stop[2] - Start[2]);

  StartView[0] := -((Stop[0] - Start[0])/MaxDist);
  StartView[1] := -((Stop[1] - Start[1])/MaxDist);
  StartView[2] := -((Stop[2] - Start[2])/MaxDist);

  StepView[0] := (2 * (Stop[0] - Start[0])/MaxDist)/GridSize;
  StepView[1] := (2 * (Stop[1] - Start[1])/MaxDist)/GridSize;
  StepView[2] := (2 * (Stop[2] - Start[2])/MaxDist)/GridSize;
end;

///////////////////////////////////////////////////////////////////////////////
function TCoordinates.IndexToViewCoord(IndexX, IndexY, indexZ: Integer): TVector3;
begin
  Result[0] := StartView[0] + IndexX * StepView[0];
  Result[1] := StartView[1] + IndexY * StepView[1];
  Result[2] := StartView[2] + IndexZ * StepView[2];
end;

///////////////////////////////////////////////////////////////////////////////
function TCoordinates.IndexToJuliaCoord(IndexX, IndexY, indexZ: Integer): TVector3;
begin
  Result[0] := Start[0] + IndexX * Step[0];
  Result[1] := Start[1] + IndexY * Step[1];
  Result[2] := Start[2] + IndexZ * Step[2];
end;

///////////////////////////////////////////////////////////////////////////////
function TCoordinates.JuliaToViewCoord(V: TVector3): TVector3;
begin
  Result[0] := StartView[0] +(V[0] - Start[0])/(Stop[0] - Start[0]) * (GridSize * StepView[0]);
  Result[1] := StartView[1] +(V[1] - Start[1])/(Stop[1] - Start[1]) * (GridSize * StepView[1]);
  Result[2] := StartView[2] +(V[2] - Start[2])/(Stop[2] - Start[2]) * (GridSize * StepView[2]);
end;

///////////////////////////////////////////////////////////////////////////////
end.
