///////////////////////////////////////////////////////////////////////////////
// Created on: April 2000
// Created by: Ronald Hordijk (hk@dgmr.nl)
//
// Some basic vector functions.
///////////////////////////////////////////////////////////////////////////////
unit vector;

interface

type
  TVector3 = array[0..2] of double;
  TVector3V = array[0..3] of double; // The first 3 value are coordinates and the third the value

function Vector3(const x, y, z: Double): TVector3; overload;
function Vector3(const V: TVector3V): TVector3; overload;
function Vector3v(const x, y, z, Value: Double): TVector3V; overload;
function Vector3v(const V: TVector3; const Value: Double): TVector3V; overload;
function GetValuePoint(const V1, V2: TVector3V; const IsoValue: Double): TVector3;
function Vector3Sub(const V1, V2: TVector3): TVector3;
function Vector3Add(const V1, V2: TVector3): TVector3;
function Vector3Length(const V: TVector3): double;
function Vector3Normalize(const V: TVector3): TVector3;
function Vector3Mult(const V: TVector3; const Mult: double): TVector3;
function Vector3Angle(const V1,V2: TVector3): double;

implementation

uses
  Math;

const
  Epsilon = 1e-7;

///////////////////////////////////////////////////////////////////////////////
function Vector3(const x, y, z: Double): TVector3;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := z;
end;

///////////////////////////////////////////////////////////////////////////////
function Vector3(const V: TVector3V): TVector3;
begin
  Result[0] := V[0];
  Result[1] := V[1];
  Result[2] := V[2];
end;

///////////////////////////////////////////////////////////////////////////////
function Vector3V(const x, y, z, Value: Double): TVector3V;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := z;
  Result[3] := Value;
end;

///////////////////////////////////////////////////////////////////////////////
function Vector3V(const V: TVector3; const Value: Double): TVector3V;
begin
  Result[0] := V[0];
  Result[1] := V[1];
  Result[2] := V[2];
  Result[3] := Value;
end;

///////////////////////////////////////////////////////////////////////////////
// get het coordinates of the point between the two points where the value equals IsoValue
function GetValuePoint(const V1, V2: TVector3V; const IsoValue: Double): TVector3;
var
  f1, f2: Double;
  Diff: Double;
begin
  Diff := Abs(V1[3] - V2[3]);

  if (Diff < Epsilon) then begin
    f1 := 0.5;
    f2 := 0.5;
  end else begin
    f1 := abs(IsoValue - V2[3]) / Diff;
    f2 := abs(IsoValue - V1[3]) / Diff;
  end;

  Result[0] := V1[0] * f1 + V2[0] * f2;
  Result[1] := V1[1] * f1 + V2[1] * f2;
  Result[2] := V1[2] * f1 + V2[2] * f2;
end;

///////////////////////////////////////////////////////////////////////////////
function Vector3Sub(const V1, V2: TVector3): TVector3;
begin
  Result[0] := V1[0] - V2[0];
  Result[1] := V1[1] - V2[1];
  Result[2] := V1[2] - V2[2];
end;

///////////////////////////////////////////////////////////////////////////////
function Vector3Add(const V1, V2: TVector3): TVector3;
begin
  Result[0] := V1[0] + V2[0];
  Result[1] := V1[1] + V2[1];
  Result[2] := V1[2] + V2[2];
end;

///////////////////////////////////////////////////////////////////////////////
function Vector3Length(const V: TVector3): double;
begin
  Result := sqrt(V[0]*V[0] + V[1]*V[1] + V[2]*V[2]);
end;

///////////////////////////////////////////////////////////////////////////////
function Vector3Normalize(const V: TVector3): TVector3;
var
  l: double;
begin
  l := Vector3Length(V);

  if (l < EPSILON ) then begin
    Result[0] := 0;
    Result[1] := 0;
    Result[2] := 1;
  end else begin
    Result[0] := V[0]/l;
    Result[1] := V[1]/l;
    Result[2] := V[2]/l;
  end;
end;

///////////////////////////////////////////////////////////////////////////////
function Vector3Mult(const V: TVector3; const Mult: double): TVector3;
begin
 Result[0] := V[0] * Mult;
 Result[1] := V[1] * Mult;
 Result[2] := V[2] * Mult;
end;

///////////////////////////////////////////////////////////////////////////////
function Vector3Angle(const V1, V2: TVector3): Double;
var
  la,lb: double;
begin
  Result := 0;

  la := Vector3Length(V1);
  lb := Vector3Length(V2);

  if (la < Epsilon) or (lb < epsilon) then
    Exit;

  Result := arccos((V1[0] * V2[0] + V1[1] * V2[1] + V1[2] * V2[2])/(la * lb));
end;

///////////////////////////////////////////////////////////////////////////////
end.
